DEFAULT_CONFIG = {
    "SQLALCHEMY_CONN": "sqlite:////tmp/chives.sqlite",
    "SQLALCHEMY_ECHO": False,
    "RABBITMQ_HOST": "localhost",
    "RABBITMQ_PORT": 5672,
    "RABBITMQ_VHOST": "/",
    "RABBITMQ_LOGIN": "guest",
    "RABBITMQ_PASSWORD": "guest",
    "MATCHING_ENGINE_DRY_RUN": False,
    "SECRET_KEY": "dev"
}