{% extends "exchange/base.html" %}
{% block content %}
<div style="display: flex; flex-direction: column">
  <div style="flex: 1 1 200px; border: 1px solid #cccccc; border-radius: 25px; display: flex; flex-direction: column; align-items: center; justify-content: center; margin-bottom: 2rem; padding: 2rem;"> <!--Net worth wrapper-->
    <span style="font-size: 2rem;">Oh no! X(</span>
    <span style="font-size: 4rem;">{{ error_msg }}</span>
  </div>
</div>
{% endblock content %}